require 'fuelcell/parser/opt_name_handler'

module Fuelcell
  module Action
    # Responsible for defining all the information about an option so that the
    # parser will be able to find it and its values when processing the command
    # line.
    class OptDefinition

      include Action::Callable
      NO_DEFAULT_ASSIGNED = '__fuelcell_no_default_assigned__'
      attr_reader :name, :long, :short, :type, :default, :names, :required,
        :cli_labels, :cli_names, :flag, :banner, :global, :cmd_path

      alias_method :flag?, :flag
      alias_method :required?, :required
      alias_method :global?, :global

      def initialize(text, config = {})
        initialize_names(text)
        @flag     = config[:flag] == true ? true : false
        @required = config[:required] == true ? true : false
        @global   = config[:global] == true ? true : false
        @type     = validate_type(config[:type])
        @default  = config.fetch(:default) { NO_DEFAULT_ASSIGNED }
        @banner   = config[:banner] || ''
        @cmd_path = config[:cmd_path]
        callable config[:run] if config.key?(:run)
      end

      def default?
        @default == NO_DEFAULT_ASSIGNED ? false : true
      end

      def name?(value)
        names.include?(value)
      end

      def cmd_path?
        !cmd_path.nil?
      end

      private

      def initialize_names(text)
        @name, @long, @short = Parser::OptNameHandler.new.call(text)
        @cli_labels = []
        @names      = [name]
        initialize_short_name(short) if short
        initialize_long_name(long)   if long
        @cli_names = cli_labels.join(' ')
      end

      def initialize_long_name(long_name)
        label = "--#{long_name}"
        @cli_labels << label
        @names += [long_name, label]
      end

      def initialize_short_name(short_name)
        label = "-#{short_name}"
        @cli_labels << label
        @names += [short_name, label]
      end

      def validate_type(value)
        case value.to_s.to_sym
        when :string, :text, :"" then :string
        when :number, :numeric   then :numeric
        when :boolean, :bool     then :bool
        when :hash               then :hash
        when :array              then :array
        else
          fail ArgumentError, "invalid type #{value}"
        end
      end
    end
  end
end
