module Fuelcell
  module Action
    # Used by the Command to manage adding option from its dsl. It is also
    # used during option parsing to find options, add global options, or check
    # if any required options have been missed
    class OptsManager
      attr_reader :options

      def initialize
        @options = {}
      end

      def each
        options.each {|_, option| yield option }
      end

      def callable?
        !callable.nil?
      end

      # Check to determine if any of the options are callable, meaning do
      # they point to another command or have a lambda to be executed
      #
      # @return [Hash]
      def callable
        list = options.select { |_, opt| opt.callable? || opt.cmd_path? }
        _, value = list.first
        value
      end

      def required_opts
        options.select { |_, opt| opt.required? }
      end

      def globals
        options.select { |_, opt| opt.global? }
      end

      def missing_opts(names)
        missing = required_opts.keys - names
        return [] if missing.empty?

        list = options.select { |(key, _)| missing.include?(key) }.values
        yield list if block_given?
        list
      end

      def add(option, config = {})
        option = create(option, config) if option.is_a?(String)
        if options.key?(option.name)
          fail "can not add option: duplicate exists with name #{option.name}"
        end
        options[option.name] = option
      end
      alias_method :opt, :add

      def remove(name)
        # looks like an option definition so lets use it's name
        if name.respond_to?(:name) && options.key?(name.name)
          return options.delete(name.name)
        end

        opt = find(name)
        return false unless opt

        options.delete(opt.name)
      end

      def find(name)
        target = false
        options.each do |(_, option)|
          target = option if option.name?(name)
        end
        target
      end
      alias_method :find_opt, :find
      alias_method :[], :find

      def create(name, config = {})
        OptDefinition.new(name, config)
      end
    end
  end
end
